package org.processmining.filterd.filters;
import java.util.List;

import org.deckfour.xes.model.XLog;
import org.processmining.filterd.parameters.Parameter;

public abstract class Filter {

	private final String filterName;
	
	Filter(String filterName) {
		this.filterName = filterName;
	}
	
	/**
	 * 
	 * @param log to be filtered
	 * @param parameters list containing objects of classes that extend the type Parameter
	 * @pre log != null \and parameters contain all the necessary ones 
	 * @return filtered log
	 */
	public abstract XLog filter(XLog log, List<Parameter> parameters);
	
	public XLog filter(XLog log, List<Parameter> parameters, boolean doTime) {
		long millis = 0L;
		if (doTime) {
			System.out.println("[Filter] " + filterName + ": started.");
			millis = System.currentTimeMillis();
		}
		XLog filteredLog = filter(log, parameters);
		if (doTime) {
			millis = System.currentTimeMillis() - millis;
			System.out.println("[Filter] " + filterName + ": finished in " + millis + " milliseconds.");
		}
		return filteredLog;
	}
	/**
	 * 
	 * @param parameters list containing objects of classes that extend the type Parameter
	 * @param whichParameter containing the name of the parameter
	 * @return first retrieved parameter object with the specified name or null
	 * 
	 */
	public Parameter getParameter(List<Parameter> parameters, String whichParameter) {
			
		for(Parameter parameter :parameters ) {
			//check for each parameter whether its name equals the desired value
			if (parameter.getName().equals(whichParameter)) {
				return parameter;
			}
		}
		return null;
	}
}
