package org.processmining.filterd.filters;

import java.util.ArrayList;
import java.util.List;

import org.deckfour.xes.classification.XEventAttributeClassifier;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.filterd.parameters.Parameter;
import org.processmining.filterd.parameters.ParameterMultipleFromSet;

public class FilterdAttributeComposeCategoricalFilter extends Filter {
	
	public static final String NAME = "Compose Categorical Attribute";

	public FilterdAttributeComposeCategoricalFilter() {
		super(NAME);
	}
	
	public XLog filter(XLog log, List<Parameter> parameters) {

		/*
		 * Retrieve the attributes to compose.
		 */
		ParameterMultipleFromSet relevantAttributesParam = (ParameterMultipleFromSet)this
				.getParameter(parameters, "relevantAttributes");		
		ParameterMultipleFromSet relevantAttributesParam2 = (ParameterMultipleFromSet)this
				.getParameter(parameters, "relevantAttributes2");		
		List<String> relevantAttributes = new ArrayList<String>(relevantAttributesParam.getChosen());
		relevantAttributes.addAll(relevantAttributesParam2.getChosen());

		/*
		 * Construct a key for the composed attribute.
		 */
		String composedKey = "";
		String infixKey = "(";
		for (String key : relevantAttributes) {
			composedKey = composedKey + infixKey + key;
			infixKey = " AND ";
		}
		composedKey = composedKey + ")";
		
		/*
		 * Construct an initial copy of the log, without traces.
		 */
		XFactory factory = XFactoryRegistry.instance().currentDefault();
		XLog filterdLog = factory.createLog(log.getAttributes());
		filterdLog.getExtensions().addAll(log.getExtensions());
		filterdLog.getGlobalEventAttributes().addAll(log.getGlobalEventAttributes());
		// The new composed attribute will be a global event attribute.
		filterdLog.getGlobalEventAttributes().add(factory.createAttributeLiteral(composedKey, "()", null));
		filterdLog.getGlobalTraceAttributes().addAll(log.getGlobalTraceAttributes());
		filterdLog.getClassifiers().addAll(log.getClassifiers());
		// Create a classifier for the new composed attribute.
		filterdLog.getClassifiers().add(new XEventAttributeClassifier(composedKey, relevantAttributes.toArray(new String[0])));
		
		/*
		 * Filter all traces and events. Every event is extended with the new composed attribute.
		 */
		for (XTrace trace : log) {
			XTrace filterdTrace = factory.createTrace(trace.getAttributes());
			for (XEvent event : trace) {
				XEvent filterdEvent = factory.createEvent();
				for (XAttribute attribute : event.getAttributes().values()) {
					filterdEvent.getAttributes().put(attribute.getKey(), attribute);
				}
				String composedValue = "";
				String infixValue = "";
				for (String key : relevantAttributes) {
					String value = (event.getAttributes().containsKey(key) ? event.getAttributes().get(key).toString() : "");
					composedValue = composedValue + infixValue + value;
					infixValue = "+";
				}
				XAttribute composedAttribute = factory.createAttributeLiteral(composedKey, composedValue, null);
				filterdEvent.getAttributes().put(composedKey, composedAttribute);
				filterdTrace.add(filterdEvent);
			}
			filterdLog.add(filterdTrace);
		}

		return filterdLog;
	}
}
