package org.processmining.filterd.plugins;

import javax.swing.JComponent;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.filterd.configurations.NotebookConfiguration;
import org.processmining.filterd.gui.adapters.AnalysisNotebookModelAdapted;
import org.processmining.framework.plugin.ProMCanceller;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;

@Plugin( //
		name = AnalysisNotebookVisualizer.NAME, //
		parameterLabels = { "Event Log", "Analysis Notebook", "Canceller", "Configuration" }, //
		returnLabels = { "Analysis Notebook on Event Log" }, //
		returnTypes = { JComponent.class }, //
		level = PluginLevel.Regular, // Not good enough for ProM Lite yet.
		userAccessible = true, //
//		icon = "prom_duck.png", //
//		url = "https://www.win.tue.nl/~hverbeek/blog/2019/09/30/visualize-log-as-log-skeleton-5/", //
		help = "Start Analysis Notebook on Event Log" //
) //
@Visualizer
public class AnalysisNotebookVisualizer extends NotebookVisualizer {

	public final static String NAME = "Analysis Notebook on Event Log";
	
	@UITopiaVariant( //
			affiliation = UITopiaVariant.EHV, //
			author = "H.M.W. Verbeek", //
			email = "h.m.w.verbeek@tue.nl" //
	) //
	@PluginVariant( //
			variantLabel = "Start new Anaylsis Notebook on Event Log, with default configuration", //
			requiredParameterLabels = { 0, 2 } //
	) //
	public JComponent run(UIPluginContext context, final XLog log, final ProMCanceller canceller) {
		return visualize(context, log, canceller, new NotebookConfiguration());
	}

	@PluginVariant( //
			variantLabel = "Start new Analysis Notebook on Event Log, with provided configuration", //
			requiredParameterLabels = { 0, 2, 3 } //
	) //
	public JComponent run(UIPluginContext context, final XLog log, final ProMCanceller canceller, final NotebookConfiguration config) {
		return visualize(context, log, canceller, config);
	}

	@UITopiaVariant( //
			affiliation = UITopiaVariant.EHV, //
			author = "H.M.W. Verbeek", //
			email = "h.m.w.verbeek@tue.nl" //
	) //
	@PluginVariant( //
			variantLabel = "Start existing Analysis Notebook on Event Log, with default configuration", //
			requiredParameterLabels = { 1, 2 } //
	) //
	public JComponent run(UIPluginContext context, AnalysisNotebookModelAdapted notebook, final ProMCanceller canceller) {
		return visualize(context, notebook, canceller, new NotebookConfiguration());
	}

	@PluginVariant( //
			variantLabel = "Start existing Analysis Notebook on Event Log, with provided configuration", //
			requiredParameterLabels = { 1, 2, 3 } //
	) //
	public JComponent run(UIPluginContext context, AnalysisNotebookModelAdapted notebook, final ProMCanceller canceller, final NotebookConfiguration config) {
		return visualize(context, notebook, canceller, config);
	}
}
