package org.processmining.filterd.plugins;

import java.io.StringReader;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Unmarshaller;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.filterd.gui.adapters.ComputationCellModelAdapted;
import org.processmining.filterd.gui.adapters.FilterButtonAdapted;
import org.processmining.filterd.gui.adapters.FilterdAbstractConfigAdapted;
import org.processmining.filterd.gui.adapters.FilterdAbstractConfigAdapter;
import org.processmining.filterd.gui.adapters.FilterdAbstractReferencingConfigAdapted;
import org.processmining.filterd.gui.adapters.NotebookModelAdapted;
import org.processmining.filterd.gui.adapters.TextCellModelAdapted;
import org.processmining.filterd.models.Notebook;
import org.processmining.filterd.parameters.Parameter;
import org.processmining.filterd.parameters.ParameterMultipleFromSet;
import org.processmining.filterd.parameters.ParameterOneFromSet;
import org.processmining.filterd.parameters.ParameterRangeFromRange;
import org.processmining.filterd.parameters.ParameterText;
import org.processmining.filterd.parameters.ParameterValueFromRange;
import org.processmining.filterd.parameters.ParameterYesNo;

/**
 * Class representing the plug-in for importing a notebook from the workspace.
 */
public abstract class NotebookLoader {

	public NotebookModelAdapted load(UIPluginContext context, Notebook notebook, XLog log) {
		NotebookModelAdapted adaptedModel;
		try {
			// read the XML.
			// add all the classes which have a XmlRootElement annotation in the newInstance method.
			JAXBContext jaxbContext = JAXBContext.newInstance(NotebookModelAdapted.class, TextCellModelAdapted.class,
						ComputationCellModelAdapted.class, FilterButtonAdapted.class,
						FilterdAbstractConfigAdapted.class, Parameter.class, ParameterMultipleFromSet.class,
						ParameterOneFromSet.class, ParameterRangeFromRange.class, ParameterText.class,
						ParameterValueFromRange.class, ParameterYesNo.class,
						FilterdAbstractReferencingConfigAdapted.class, FilterdAbstractConfigAdapted.class); // Create JAXB Context.

			// create the JAXB unmarshaller
			Unmarshaller jaxbUnmarshaller = jaxbContext.createUnmarshaller();
			// create a string reader for the unmarshaller 
			StringReader reader = new StringReader(notebook.getContents());
			// set the initial input for the abstract configs.
			FilterdAbstractConfigAdapter.setInitialInput(log);
			adaptedModel = (NotebookModelAdapted) jaxbUnmarshaller.unmarshal(reader); // serialize the notebook from the XML input
			// embed the initial input to the adaptedModel.
			adaptedModel.setInitialInput(log);
			String label = XConceptExtension.instance().extractName(log);
			if (label == null) {
				label = "<unlabeled>";
			}
			context.getFutureResult(0).setLabel(getLabel(label));
			return adaptedModel;
		} catch (

		Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	public abstract String getLabel(String label);
}
